PROMPT
PROMPT Creating employee table...
CREATE TABLE employee (
  employee_id               NUMBER,
  employee_name             VARCHAR2(40),
  employee_hire_date        DATE DEFAULT sysdate,
  employee_termination_date DATE,
  employee_billing_rate     NUMBER,
  CONSTRAINT employee_pk
    PRIMARY KEY (employee_id)
);

CREATE INDEX employee_by_name ON employee (
    employee_name
    );

CREATE OR REPLACE FUNCTION terminate_employee
    (emp_id IN employee.employee_id%TYPE,
     emp_hire_date OUT employee.employee_hire_date%TYPE,
     emp_term_date IN employee.employee_termination_date%TYPE)
RETURN INTEGER AS
BEGIN
  UPDATE employee
     SET employee_termination_date = emp_term_date
   WHERE employee_id = emp_id;

  SELECT employee_hire_date INTO emp_hire_date
    FROM employee
   WHERE employee_id = emp_id;

  RETURN 1;
EXCEPTION
WHEN OTHERS THEN
  RETURN 0;
END;
/

CREATE OR REPLACE TRIGGER emp_hire_date_check
BEFORE INSERT OR UPDATE ON employee
FOR EACH ROW
BEGIN
  IF :NEW.employee_hire_date < TRUNC(SYSDATE - 30) THEN
    RAISE_APPLICATION_ERROR (-20000,
      'An employee''s hire date may only be backdated 30 days, and not more.');
  END IF;
END;
/

CREATE OR REPLACE TRIGGER emp_delete_check
BEFORE DELETE ON employee
FOR EACH ROW
BEGIN
  IF (:OLD.employee_termination_date IS NULL)
     OR (:OLD.employee_termination_date >= TRUNC(SYSDATE)+1) THEN
        RAISE_APPLICATION_ERROR (-20001,
          'You must terminate an employee before deleting his record.');
  END IF;
END;
/

PROMPT
PROMPT Creating project table...
CREATE TABLE project (
  project_id              NUMBER,
  project_name            VARCHAR2(40),
  project_budget          NUMBER,
  CONSTRAINT project_pk
    PRIMARY KEY (project_id)
);

PROMPT
PROMPT Creating project_hours table...
CREATE TABLE project_hours (
  project_id              NUMBER,
  employee_id             NUMBER,
  time_log_date           DATE,
  hours_logged            NUMBER,
  dollars_charged         NUMBER,
  CONSTRAINT project_hours_pk
    PRIMARY KEY (project_id, employee_id, time_log_date),
  CONSTRAINT proj_hours_fkto_employee
    FOREIGN KEY (employee_id) REFERENCES employee,
  CONSTRAINT proj_hours_fkto_project
    FOREIGN KEY (project_id) REFERENCES project
);

CREATE SYNONYM all_tables FOR sys.all_tables;
CREATE SYNONYM all_indexes FOR sys.all_indexes;
CREATE SYNONYM all_constraints FOR sys.all_constraints;

